#include <math.h>

#include <qdialog.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpixmap.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qprogressdialog.h>
#include <qtabwidget.h>
#include <qtable.h>
#include <qwidget.h>
#include <qwidgetfactory.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qspinbox.h>

#include "xpm/fileopen.xpm"
#include "xpm/fileprint.xpm"

#include "ui/dabout.h"
#include "ui/danimate.h"
#include "ui/darrowstyle.h"
#include "ui/datomstyle.h"
#include "ui/dprojvector.h"
#include "ui/dscaledisp.h"
#include "ui/dselectplots.h"

#include "application.h"
#include "messages.h"
#include "qplotwidget.h"
#include "setatomstyle.h"
#include "strings.h"
#include "ddplot.h"



//**************************************************************************************************
//  class AppWin
//**************************************************************************************************


AppWin::AppWin()
    : QMainWindow( 0, "ddplot", WDestructiveClose )
{
  QPixmap openIcon,printIcon;

  tabWidget = NULL;

  loadSettings();

  openIcon = QPixmap( fileopen );
  printIcon = QPixmap( fileprint );

  resize( WINWIDTH, WINHEIGHT );
  

  //----------------------------------------------------------------------------

  mfile = new QPopupMenu( this );
  Q_CHECK_PTR( mfile );
  menuBar()->insertItem( "&File", mfile );

  OpenID     = mfile->insertItem( openIcon, "&Open...", this, SLOT(load()), CTRL+Key_O );
  CloseID    = mfile->insertItem( "&Close", this, SLOT(close()), CTRL+Key_F3 );
  CloseAllID = mfile->insertItem( "C&lose all", this, SLOT(closeAll()) );
  mfile->insertSeparator();
  PrintID    = mfile->insertItem( printIcon, "&Print...", this, SLOT(print()), CTRL+Key_P ); 
  mfile->insertSeparator();
  ExportID   = mfile->insertItem( "&Export to JMol format...", this, SLOT(exportXYZ()) );
  AnimateID  = mfile->insertItem( "&Animate all frames to .GIF...", this, SLOT(animate()) );
  mfile->insertSeparator();
  QuitID     = mfile->insertItem( "&Quit", this, SLOT(quit()), CTRL+Key_Q );

  //----------------------------------------------------------------------------

  mview = new QPopupMenu( this );
  Q_CHECK_PTR( mview );
  menuBar()->insertItem( "&View", mview );

  ZoomInID    = mview->insertItem( "Zoom &in", this, SLOT(zoomIn()), Key_Plus );  
  ZoomOutID   = mview->insertItem( "Zoom &out", this, SLOT(zoomOut()), Key_Minus );  
  FitWinID    = mview->insertItem( "&Fit window", this, SLOT(fitWin()), Key_Asterisk );
  mview->insertSeparator();
  PanRightID  = mview->insertItem( "Pan &right", this, SLOT(panRight()), SHIFT+Key_Right );  
  PanLeftID   = mview->insertItem( "Pan &left", this, SLOT(panLeft()), SHIFT+Key_Left );   
  PanUpID     = mview->insertItem( "Pan &up", this, SLOT(panUp()), SHIFT+Key_Up );  
  PanDownID   = mview->insertItem( "Pan &down", this, SLOT(panDown()), SHIFT+Key_Down );  
  mview->insertSeparator();
  PanFRightID = mview->insertItem( "Pan fast &right", this, SLOT(panFastRight()), CTRL+Key_Right );  
  PanFLeftID  = mview->insertItem( "Pan fast &left", this, SLOT(panFastLeft()), CTRL+Key_Left );   
  PanFUpID    = mview->insertItem( "Pan fast &up", this, SLOT(panFastUp()), CTRL+Key_Up );  
  PanFDownID  = mview->insertItem( "Pan fast &down", this, SLOT(panFastDown()), CTRL+Key_Down );

  //----------------------------------------------------------------------------

  mplot = new QPopupMenu( this );
  Q_CHECK_PTR( mplot );
  menuBar()->insertItem( "&Plot", mplot );

  SelPlotsID  = mplot->insertItem( "&Select plots...", this, SLOT(selectPlots()), CTRL+Key_S );
  mplot->insertSeparator();
  UnrelViewID = mplot->insertItem( "&Unrelaxed configuration", this, SLOT(plotUnrelaxed()), Key_U );  
  RelViewID   = mplot->insertItem( "&Relaxed configuration", this, SLOT(plotRelaxed()), Key_R );
  CompViewID  = mplot->insertItem( "&Composite view (relaxed+unrelaxed)", this, SLOT(plotComposite()), Key_C );
  mplot->insertSeparator();
  MagDispID   = mplot->insertItem( "P&rint magnitudes of displacements", this, SLOT(printMagDisp()), Key_M );
  NumAtomID   = mplot->insertItem( "Pr&int numbers of atoms", this, SLOT(printAtomNum()), Key_N );
  mplot->insertSeparator();
  EdgeCompID  = mplot->insertItem( "&Edge component", this, SLOT(plotEdgeComponent()), Key_E );  
  ScrewCompID = mplot->insertItem( "&Screw component", this, SLOT(plotScrewComponent()), Key_S );
  mplot->insertSeparator();
  DAtLayersID = mplot->insertItem( "&Distinguish atomic layers", this, SLOT(plotAtomLayers()), Key_L );  
  DAtTypesID  = mplot->insertItem( "D&istinguish atom types", this, SLOT(plotAtomTypes()), Key_T );  
  mplot->insertSeparator();
  DCenterID   = mplot->insertItem( "S&how dislocation center", this, SLOT(disloCenter()) );

  mplot->setItemChecked( UnrelViewID, ATOMPOS==UNRELAXED );
  mplot->setItemChecked( RelViewID, ATOMPOS==RELAXED );
  mplot->setItemChecked( CompViewID, ATOMPOS==COMPOSITE );

  mplot->setItemChecked( MagDispID, PRINT_MAG_DISP );

  mplot->setItemChecked( EdgeCompID, DISP_COMPONENT==EDGE );
  mplot->setItemChecked( ScrewCompID, DISP_COMPONENT==SCREW );

  mplot->setItemChecked( DAtLayersID, PLOT_TYPE==PLOT_ATOM_LAYERS );
  mplot->setItemChecked( DAtTypesID, PLOT_TYPE==PLOT_ATOM_TYPES );

  mplot->setItemChecked( DCenterID, SHOW_DCENTER );

  //----------------------------------------------------------------------------

  moper = new QPopupMenu( this );
  Q_CHECK_PTR( moper );
  menuBar()->insertItem( "&Operations", moper );

  ProjVectID  = moper->insertItem( "Set &projection vector...", this, SLOT(projVector()), Key_P );  
  ScaleDispID = moper->insertItem( "&Scale displacements...", this, SLOT(scaleDisp()) );

  //----------------------------------------------------------------------------

  mopts = new QPopupMenu( this );
  Q_CHECK_PTR( mopts );
  menuBar()->insertItem( "&Appearance", mopts );

  CrystPlotID = mopts->insertItem( "&Crystal structure plotting...", this, SLOT(crystalPlot()) );
  ArrPlotID   = mopts->insertItem( "&Arrows plotting...", this, SLOT(arrowsPlot()) );

  //----------------------------------------------------------------------------

  mhelp = new QPopupMenu( this );
  Q_CHECK_PTR( mhelp );
  menuBar()->insertItem( "&Help", mhelp );

  AboutID = mhelp->insertItem( "&About...", this, SLOT(about()), CTRL+Key_A );

  //----------------------------------------------------------------------------

  enableMenuItems(false);

  printer = new QPrinter( QPrinter::PrinterResolution );
  printer->setColorMode(QPrinter::Color);
  printer->setPageSize(QPrinter::Letter);

  // loading of the plot file entered through the command-line
  if (PLOT_FILE!="")
    loadPlotFile( PLOT_FILE );
}


AppWin::~AppWin()
{
  delete( printer );
}


void AppWin::about()
{
  DAbout *dialog = new DAbout( this, "", true, WShowModal );
  dialog->exec();
  delete( dialog );
}


void AppWin::close()
{
  int NCurrentPlot = tabWidget->currentPageIndex();
  int NPlots = tabWidget->count();
  int i;


  //  tabWidget->removePage( plotWidget[NCurrentPlot] );
  tabWidget->removePage( plotWidget[NCurrentPlot] );
  delete( plotWidget[NCurrentPlot] );

  for (i=NCurrentPlot; i<NPlots; i++)
    plotWidget[i] = plotWidget[i+1];

  // the deleted plot was the last one
  if (NCurrentPlot>=tabWidget->count())
    NCurrentPlot = tabWidget->count()-1;

  if (tabWidget->count()==0) {
    delete( tabWidget );
    tabWidget = NULL;
    enableMenuItems(false);
  } else {
    tabWidget->showPage( plotWidget[NCurrentPlot] );
    repaintStatusBar();
  }
}


void AppWin::closeAll()
{
  int n;
  int NPlots = tabWidget->count();

  for (n=0; n<NPlots; n++) {
    tabWidget->removePage( plotWidget[n] );
    delete( plotWidget[n] );
  }

  delete( tabWidget );
  tabWidget = NULL;

  repaintStatusBar();
  enableMenuItems(false);
}


void AppWin::enableMenuItems( bool flag )
{
  mfile->setItemEnabled( CloseID, flag );
  mfile->setItemEnabled( CloseAllID, flag );
  mfile->setItemEnabled( PrintID, flag );
  mfile->setItemEnabled( ExportID, flag );
  mfile->setItemEnabled( AnimateID, flag );
    
  mview->setItemEnabled( ZoomInID, flag );
  mview->setItemEnabled( ZoomOutID, flag );
  mview->setItemEnabled( FitWinID, flag );
  mview->setItemEnabled( PanRightID, flag );
  mview->setItemEnabled( PanLeftID, flag );
  mview->setItemEnabled( PanUpID, flag );
  mview->setItemEnabled( PanDownID, flag );
  mview->setItemEnabled( PanFRightID, flag );
  mview->setItemEnabled( PanFLeftID, flag );
  mview->setItemEnabled( PanFUpID, flag );
  mview->setItemEnabled( PanFDownID, flag );

  mplot->setItemEnabled( SelPlotsID, flag );
  mplot->setItemEnabled( UnrelViewID, flag );
  mplot->setItemEnabled( RelViewID, flag );
  mplot->setItemEnabled( CompViewID, flag );
  mplot->setItemEnabled( MagDispID, flag );
  mplot->setItemEnabled( NumAtomID, flag );
  mplot->setItemEnabled( EdgeCompID, flag );
  mplot->setItemEnabled( ScrewCompID, flag );
  mplot->setItemEnabled( DAtLayersID, flag );
  mplot->setItemEnabled( DAtTypesID, flag );
  mplot->setItemEnabled( DCenterID, flag );

  moper->setItemEnabled( ProjVectID, flag );
  moper->setItemEnabled( ScaleDispID, flag );

  mopts->setItemEnabled( CrystPlotID, flag );
  mopts->setItemEnabled( ArrPlotID, flag );
}


void AppWin::loadPlotFile( QString fn )
{
  int pos, NCurrentPlot;


  if (tabWidget==NULL) {
    tabWidget = new QTabWidget( this, "tabWidget" );
    tabWidget->setGeometry( 0, 23, width(), height()-40 );  
  }

  NCurrentPlot = tabWidget->count();

  plotWidget[NCurrentPlot] = new QPlotWidget( tabWidget );
  plotWidget[NCurrentPlot]->LoadFile( fn );
  plotWidget[NCurrentPlot]->PrepareBlock();
  plotWidget[NCurrentPlot]->SetGeometry();

  // only the file name is printed
  pos = fn.findRev('/');  
  if (pos>=0)
    fn.remove(0,pos+1);
    
  tabWidget->addTab( plotWidget[NCurrentPlot], fn );
  tabWidget->showPage( plotWidget[NCurrentPlot] );
  tabWidget->show();

  repaintStatusBar();
  enableMenuItems(true);
}


void AppWin::load()
{
  QStringList files = QFileDialog::getOpenFileNames( "* - plot files generated by SBOP", 
    "", this, "OpenFileDialog", "Select one of more plot files to open" );

  if (!files.isEmpty()) {
    QStringList::Iterator fn = files.begin();

    while( fn != files.end() ) {
      loadPlotFile(*fn);
      ++fn;
    }
  }
}


void AppWin::exportXYZ()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];
  char * msg;
  FILE *f;
  const char *xyzfile;
  int n;

  xyzfile = pw->FName.append(".xyz").ascii();

  f = fopen(xyzfile, "w+");
  fprintf(f,"%d\n",pw->NRel);
  fprintf(f,"Place a comment here\n");
  for (n=0; n<pw->NRel; n++)
    fprintf(f,"%s  %12.5lf  %12.5lf  %12.5lf\n",
      ATOM_NAME[pw->atomType[n]].str(),pw->xyzRel[n][0],pw->xyzRel[n][1],pw->xyzRel[n][2]);
  fclose(f);

  msg = "Atomic data successfully saved to file <name>.XYZ";
  msgInfo(msg);  
}


void AppWin::print()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];

  if (printer->setup(this))
    pw->printIt(printer);
}


void AppWin::resizeEvent( QResizeEvent * )
{
  if (tabWidget==NULL)
    return;

  int NCurrentPlot = tabWidget->currentPageIndex();

  if (tabWidget!=NULL) {
    tabWidget->resize( width(), height()-40 ); 
    plotWidget[NCurrentPlot]->resize( width(), height()-63 );
    plotWidget[NCurrentPlot]->repaint();
  }
}


void AppWin::closeEvent( QCloseEvent * )
{
  quit();
}


void AppWin::quit()
{
  //  if (msgWarning( warn_ReallyQuit ))
  exit(0);
}


//------------------------------------------------------------------------------
//  SLOTS
//------------------------------------------------------------------------------


void AppWin::animate()
{
  QString outf,cmd,args,animFile,msg;
  int n,nbefore;
  int NPlots = tabWidget->count();
  int density,delay,nloops;
  bool exit;


  // checking to see if ImageMagick is present
  exit = system("convert -version")==-1;
  if (exit) {
    msgError(err_NoImageMagick);    
    return;
  }

  DAnimate *dialog = new DAnimate( this, "", true, WShowModal );
  dialog->exec();
  
  if (!dialog->result()==QDialog::Accepted)
    return;

  animFile = dialog->animFile->text();
  density = dialog->density->value();
  delay = dialog->delay->value();
  nloops = dialog->nloops->value();
  args = dialog->args->text();

  // collecting the data

  system("rm *.#af");

  nbefore = tabWidget->currentPageIndex();
  printer->setColorMode(QPrinter::Color);

  QProgressDialog pdlg( "Collecting plots...", "&Abort", NPlots-1,
			this, "progress", TRUE );
  pdlg.setMinimumDuration(0);
  pdlg.setAutoClose(FALSE);
  pdlg.setAutoReset(FALSE);
  pdlg.show();

  exit = false;

  for (n=0; n<NPlots; n++) {
    if (!plotWidget[n]->isSelected)
      continue;

    outf.sprintf( "%03d.#af", n);
    printer->setOutputToFile(true);
    printer->setOutputFileName( outf );
    
    tabWidget->setCurrentPage(n);
    plotWidget[n]->printIt(printer);

    pdlg.setProgress(n);
    pdlg.repaint();

    if (pdlg.wasCancelled()) {  
      exit = true;
      n = NPlots-1;
    }
  }

  printer->setOutputToFile( false );
  tabWidget->setCurrentPage( nbefore );
  plotWidget[nbefore]->repaint();

  if (exit)
    return;

  // calling the ImageMagick

  pdlg.setLabelText("Generating animation... (this will take a while)");
  pdlg.setProgress(NPlots-1);
  pdlg.repaint();

  cmd.sprintf("convert -density %dx%d -delay %d %s *.#af %s",
	      density,density,delay,args.latin1(),animFile.latin1());
  system(cmd.latin1());
  system("rm *.#af");

  msg.sprintf(info_AnimationDone,animFile.latin1());
  QMessageBox::information( 0, "Information", msg );
}


void AppWin::arrowsPlot()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];
  int NPlots = tabWidget->count();
  int n;
  bool ok;

  DArrowStyle *dialog = new DArrowStyle( this, "", true, WShowModal );
  dialog->ithickness->setValue( pw->thicknessArrow );
  dialog->ishortest->setValue( pw->shortestArrow );

  dialog->exec();
  ok = dialog->result()==QDialog::Accepted;

  if (ok) {
    for (n=0; n<NPlots; n++) {
      pw = plotWidget[n];
      if (!pw->isSelected)
	continue;

      pw->thicknessArrow = dialog->ithickness->text().toInt();
      pw->shortestArrow = dialog->ishortest->text().toInt();
      pw->repaint();
    }
  }
}


void AppWin::crystalPlot()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];
  QPixmap pix;
  int NPlots = tabWidget->count();
  int i,n,nrows;
  bool ok;

  // declaration
  SetAtomStyle *astyle = new SetAtomStyle(pw);
  astyle->setData();

  // execution
  astyle->exec();
  ok = astyle->result()==QDialog::Accepted;

  // change of settings
  if (ok) {
    ok = astyle->verify();
    if (ok) {
      for (n=0; n<NPlots; n++) {
	pw = plotWidget[n];
	if (!pw->isSelected)  
	  continue;

	for (i=0; i<=pw->NumZLayers; i++) {
	  pw->zDiamLayer[i] = astyle->atomTable->text(i,1).toInt();
	  pw->zLineThickLayer[i] = astyle->atomTable->text(i,2).toInt();
	  
	  pw->zColorLayer[i][0] = astyle->zColor[i][0];
	  pw->zColorLayer[i][1] = astyle->zColor[i][1];
	}
	pw->repaint();    
      }
    }
  }
}


void AppWin::disloCenter()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];

  SHOW_DCENTER = !SHOW_DCENTER;
  mplot->setItemChecked( DCenterID, SHOW_DCENTER );
  pw->DisloCenter = SHOW_DCENTER; 
  pw->repaint();
  repaintStatusBar();
}



void AppWin::fitWin()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->xPan = 0;
  pw->yPan = 0;
  pw->ZFact = 1;
  pw->repaint();

  repaintStatusBar();
}


void AppWin::interpretCommand(TString &cmd)
{
  TString params[10];
  int npar=0,idx,i,j,id;
  bool alldone=false;

  // get rid of comments
  cmd.ShrinkToEnd('#');
  cmd.ShrinkToEnd('\n');
  cmd.ShrinkToEnd('\r');   // Linux terminates with \r\n

  if (cmd=="")
    return;

  do {
    idx = cmd.Pos(',');
    if (idx==-1) {                               // last part of the command
      npar++;
      params[npar-1] = cmd;
      alldone = true;
    } else {                                     // select the parts of the command
      npar++;
      params[npar-1].Copy(cmd,0,idx);
      cmd.LFrac(idx);
    }
    params[npar-1].LTrim();                      // get rid of spaces
    params[npar-1].RTrim();
  } while (!alldone);
  params[0].UpStr();

  //-------------------------------------------
  // interpretation of the commands read above
  //-------------------------------------------

  if (params[0]=="ATOM.DIAMETER") {
    for (i=1; i<=npar-1; i++)
      ATOM_DIAM[i-1] = params[i].ToInt();
    NUM_DIAM = npar-1;
  }

  if (params[0]=="ATOM.THICKNESS") {
    for (i=1; i<=npar-1; i++)
      ATOM_THICKNESS[i-1] = params[i].ToInt();
    NUM_THICKNESS = npar-1;
  }

  if (params[0]=="ATOM.FGCOLOR") {
    for (i=1; i<=npar-1; i++)
      ATOM_FGCOLOR[i-1] = QColor(params[i].str());
    NUM_FGCOLORS = npar-1;
  }

  if (params[0]=="ATOM.BGCOLOR") {
    for (i=1; i<=npar-1; i++)
      ATOM_BGCOLOR[i-1] = QColor(params[i].str());
    NUM_BGCOLORS = npar-1;
  }

  if (params[0]=="ANAME") {
    id = params[1].ToInt();
    ATOM_NAME[id] = params[2];
  }

  if (params[0]=="ATOMNUMBERS") {
    ATOM_NUMBERS = (params[1]=="on");
  }
      
  if (params[0]=="ADISTINGUISH") {
    if (params[1]=="layer")
      PLOT_TYPE = PLOT_ATOM_LAYERS;

    if (params[1]=="type")
      PLOT_TYPE = PLOT_ATOM_TYPES;
  }

  if (params[0]=="ARROW.SHORTEST")          
    ARROW_SHORTEST = params[1].ToInt();

  if (params[0]=="ARROW.THICKNESS")          
    ARROW_THICKNESS = params[1].ToInt();

  if (params[0]=="ATOMPOS") {
    if (params[1]=="relaxed")
      ATOMPOS = RELAXED;

    if (params[1]=="unrelaxed")
      ATOMPOS = UNRELAXED;

    if (params[1]=="composite")
      ATOMPOS = COMPOSITE;

    if (params[1]=="relaxed" or params[1]=="composite") {
      if (npar>2)
        ATOMDISP_SCALE = params[2].ToInt();
      else
        ATOMDISP_SCALE = 1.0;
    }
  }
   

  if (params[0]=="CAXES") {
    for (i=0; i<3; i++) {
      for (j=0; j<3; j++) {
        idx = 3*i+j+1;
        CAXES[i][j] = params[idx].ToDouble();
      }
    }
  }

  if (params[0]=="DISPCOMPONENT") {
    if (params[1]=="edge")  
      DISP_COMPONENT = EDGE;

    if (params[1]=="screw")
      DISP_COMPONENT = SCREW;
  }

  if (params[0]=="DSCALEFACT") {
    if (params[1]=="auto")
      DSCALE_FACT = AUTO;
    else
      DSCALE_FACT = params[1].ToDouble();
  }

  if (params[0]=="PRINTMAGDISP") {
    if (params[1]=="on")
      PRINT_MAG_DISP = true;

    else
      PRINT_MAG_DISP = false;
  }

  if (params[0]=="PROJVECTOR") {
    PROJ_VECTOR[0] = params[1].ToDouble();
    PROJ_VECTOR[1] = params[2].ToDouble();
    PROJ_VECTOR[2] = params[3].ToDouble(); 
  }

  if (params[0]=="WINSIZE") {
    WINWIDTH = params[1].ToInt();
    WINHEIGHT = params[2].ToInt();
  }

  if (params[0]=="ZOOMFACT") {
    ZOOM_FACT = params[1].ToDouble();
  }
}


void AppWin::loadSettings()
{
  TString tbuf;
  FILE *f;
  char *buf;
  bool eof;

  f = fopen( SETTINGS_FILE, "r+" );
  if (f!=NULL) {
    if (STARTUP_DEBUG==ON)
      printf("Reading settings from file %s\n",SETTINGS_FILE.latin1());
  } else {
    if (STARTUP_DEBUG==ON)
      printf("Warning:  No configuration file $HOME/.ddplot found. Default settings will be used.\n");
    return;
  }

  buf = (char *) malloc(128);

  while (!feof(f)) {
    // read whole line
    eof = fgets(buf,128,f)==NULL;
    if (eof)
      break;

    tbuf = buf;
    interpretCommand(tbuf);
  }

  free(buf);
  fclose(f);
}


void AppWin::panRight()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(PAN_NORMAL,0);
  repaintStatusBar();
}


void AppWin::panLeft()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(-PAN_NORMAL,0);
  repaintStatusBar();
}


void AppWin::panUp()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(0,PAN_NORMAL);
  repaintStatusBar();
}


void AppWin::panDown()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(0,-PAN_NORMAL);
  repaintStatusBar();
}


void AppWin::panFastRight()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(PAN_FAST,0);
  repaintStatusBar();
}


void AppWin::panFastLeft()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(-PAN_FAST,0);
  repaintStatusBar();
}


void AppWin::panFastUp()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(0,PAN_FAST);
  repaintStatusBar();
}


void AppWin::panFastDown()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Pan(0,-PAN_FAST);
  repaintStatusBar();
}


void AppWin::plotAtomLayers()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->plotType = PLOT_ATOM_LAYERS;
  pw->repaint();

  mplot->setItemChecked( DAtLayersID, true );
  mplot->setItemChecked( DAtTypesID, false );
}


void AppWin::plotAtomTypes()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];
  
  pw->plotType = PLOT_ATOM_TYPES;
  pw->repaint();

  mplot->setItemChecked( DAtLayersID, false );
  mplot->setItemChecked( DAtTypesID, true );
}


void AppWin::plotEdgeComponent()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->DispComponent = EDGE;
  pw->repaint();
  repaintStatusBar();

  mplot->setItemChecked( EdgeCompID, true);
  mplot->setItemChecked( ScrewCompID, false);
}

void AppWin::plotScrewComponent()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->DispComponent = SCREW;
  pw->repaint();
  repaintStatusBar();

  mplot->setItemChecked( EdgeCompID, false);
  mplot->setItemChecked( ScrewCompID, true);
}


void AppWin::plotUnrelaxed()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->AtomPos = UNRELAXED;
  pw->repaint();
  repaintStatusBar();

  mplot->setItemChecked( UnrelViewID, true );
  mplot->setItemChecked( RelViewID, false );
  mplot->setItemChecked( CompViewID, false );
}


void AppWin::plotRelaxed()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->AtomPos = RELAXED;
  pw->repaint();
  repaintStatusBar();

  mplot->setItemChecked( UnrelViewID, false );
  mplot->setItemChecked( RelViewID, true );
  mplot->setItemChecked( CompViewID, false );
}


void AppWin::plotComposite()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->AtomPos = COMPOSITE;
  pw->repaint();
  repaintStatusBar();

  mplot->setItemChecked( UnrelViewID, false );
  mplot->setItemChecked( RelViewID, false );
  mplot->setItemChecked( CompViewID, true );
}


void AppWin::printMagDisp()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->PrintMagDisp = abs(pw->PrintMagDisp-1); 
  pw->repaint();

  mplot->setItemChecked( MagDispID, pw->PrintMagDisp );
}


void AppWin::printAtomNum()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];

  pw->AtomNumbers = abs(pw->AtomNumbers-1); 
  pw->repaint();

  mplot->setItemChecked( NumAtomID, pw->AtomNumbers );
}


void AppWin::projVector()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];
  double coord[3];
  int NPlots = tabWidget->count();
  int i,n;
  bool ok,okx,oky,okz;

  do {
    // dialog may not be WDestructiveClose, because it would delete all data inside exec()
    DProjVector *dialog = new DProjVector( this, "", true, WShowModal );

    dialog->xcoord->insert( QString::number( pw->ProjVector[0] ));
    dialog->ycoord->insert( QString::number( pw->ProjVector[1] ));
    dialog->zcoord->insert( QString::number( pw->ProjVector[2] ));

    dialog->exec();
    ok = dialog->result()==QDialog::Accepted;

    if (ok) {
      coord[0] = dialog->xcoord->text().toDouble( &okx );
      coord[1] = dialog->ycoord->text().toDouble( &oky );
      coord[2] = dialog->zcoord->text().toDouble( &okz );

      if (okx && oky && okz) {
	for (n=0; n<NPlots; n++) {
	  pw = plotWidget[n];
	  if (!pw->isSelected) 
	    continue;

	  for (i=0; i<3; i++)
	    pw->ProjVector[i] = coord[i];

	  pw->DispComponent = MIXED;
	  repaintStatusBar();
	  pw->repaint();
	}
      } else
        msgError(err_ProjVectIsString);
    }
  } while ( ok && (!okx || !oky || !okz) );
}


void AppWin::repaintStatusBar()
{
  if (tabWidget==NULL)
    return;

  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];
  char *dcomp;
  char msg[128];
  int NPlots = tabWidget->count();

  switch(pw->DispComponent) {
    case EDGE:
      dcomp = "EDGE";
      break;
    case SCREW:
      dcomp = "SCREW";
      break;
    default:
      dcomp = "MIXED";
  }

  sprintf(msg, "%d plot(s) | MAG=%0.2lf | DSCALE=%0.2lf | PANX=%0.2lf, PANY=%0.2lf" \
    " | PROJVECT=[%0.1lf,%0.1lf,%0.1lf] | DCOMP=%s | ATOMS=%s", 
    NPlots, 
    pw->ZFact,
    pw->DScaleFact,
    pw->xPan,
    pw->yPan,
    pw->ProjVector[0],
    pw->ProjVector[1],
    pw->ProjVector[2],
    dcomp,
    ATOMPOS_NAME[pw->AtomPos]);

  statusBar()->message( msg, 0 );
}


void AppWin::scaleDisp()
{
  QPlotWidget *pw = plotWidget[ tabWidget->currentPageIndex() ];
  double sfact;
  int NPlots = tabWidget->count();
  int n;
  bool ok,okk;

  DScaleDisp *dialog = new DScaleDisp( this, "", true, WShowModal );
  dialog->sfactor->insert( QString::number( pw->DScaleFact ));
  do {
    dialog->exec();
    ok = dialog->result()==QDialog::Accepted;

    if (ok) {
      sfact = dialog->sfactor->text().toDouble( &okk );
      if (okk) {
	for (n=0; n<NPlots; n++) {
	  pw = plotWidget[n];
	  if (!pw->isSelected) 
	    continue;

	  pw->DScaleFact = sfact; 
	  pw->repaint();
	}
      } else
        msgError(err_ScaleDispIsString);   
    }
  } while (ok && !okk);

  repaintStatusBar();
}


void AppWin::selectPlots()
{
  QString item;
  int sigSelAll=100, sigUnselAll=101;
  int pos,res;
  unsigned int i;
  bool exit,selFlag;


  DSelectPlots *dialog = new DSelectPlots( this, "", true, WShowModal );

  // set up the data
  for (i=0; i<tabWidget->count(); i++) {
    item = plotWidget[i]->FName;
    
    // only the file name is printed
    pos = item.findRev('/');  
    if (pos>=0)
      item.remove(0,pos+1);
    
    selFlag = plotWidget[i]->isSelected;

    dialog->plotFiles->insertItem(item);
    dialog->plotFiles->setSelected(i,selFlag);
  }

  do {
    // get the user's response
    dialog->show();
    dialog->exec();
    res = dialog->result();

    exit = true;

    // clicked on Select All
    if (res==sigSelAll) {
      for (i=0; i<dialog->plotFiles->count(); i++)
	dialog->plotFiles->setSelected(i,true);
      exit = false;
    }

    // clicked on Unselect All
    if (res==sigUnselAll) {
      for (i=0; i<dialog->plotFiles->count(); i++)
	dialog->plotFiles->setSelected(i,false);
      exit = false;
    }

    // clicked OK
    if (res==QDialog::Accepted) {
      for (i=0; i<dialog->plotFiles->count(); i++) {
	selFlag = dialog->plotFiles->isSelected(i);
	plotWidget[i]->isSelected = selFlag;
	tabWidget->setTabEnabled( plotWidget[i], selFlag );
      }

      tabWidget->repaint();
      repaintStatusBar();
    }
  } while (!exit);
}


void AppWin::zoomIn()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Zoom(2.0); 
  repaintStatusBar();
}


void AppWin::zoomOut()
{
  plotWidget[ tabWidget->currentPageIndex() ]->Zoom(0.5); 
  repaintStatusBar();
}
