#ifndef APPLICATION_H
#define APPLICATION_H

#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qtabwidget.h>

#include "qplotwidget.h"
#include "strings.h"


class AppWin: public QMainWindow
{
  Q_OBJECT

public:
  QPrinter *printer;
  QTabWidget *tabWidget;
  QPlotWidget *plotWidget[MAX_PLOTS];

  QPopupMenu *mfile, *mview, *mplot, *moper, *mopts, *mhelp;

  int OpenID, CloseID, CloseAllID, PrintID, ExportID, AnimateID, QuitID;
  int ZoomInID, ZoomOutID, FitWinID, PanRightID, PanLeftID, PanUpID, PanDownID;
  int SelPlotsID, PanFRightID, PanFLeftID, PanFUpID, PanFDownID;
  int UnrelViewID, RelViewID, CompViewID, MagDispID, NumAtomID, EdgeCompID;
  int ScrewCompID, DAtLayersID, DAtTypesID, DCenterID;
  int ProjVectID, ScaleDispID;
  int CrystPlotID, ArrPlotID, AboutID;

  AppWin();
  ~AppWin();

protected:
  void closeEvent( QCloseEvent* );
  virtual void resizeEvent( QResizeEvent * );

private slots:
  void about();
  void close();
  void closeAll();
  void loadPlotFile( QString fn );
  void load();
  void enableMenuItems( bool flag );
  void exportXYZ();
  void print();
  void quit();

  void animate();
  void arrowsPlot();
  void crystalPlot();
  void disloCenter();
  void fitWin();
  void interpretCommand(TString &cmd);
  void loadSettings();
  void panRight();
  void panLeft();
  void panUp();
  void panDown();
  void panFastRight();
  void panFastLeft();
  void panFastUp();
  void panFastDown();
  void plotAtomLayers();
  void plotAtomTypes();
  void plotEdgeComponent();
  void plotScrewComponent();
  void plotUnrelaxed();
  void plotRelaxed();
  void plotComposite();
  void printMagDisp();
  void printAtomNum();
  void projVector();
  void repaintStatusBar();
  void selectPlots();
  void scaleDisp();
  void zoomIn();  
  void zoomOut();
};

#endif
