#ifndef QPLOTWIDGET_H
#define QPLOTWIDGET_H

#include <qwidget.h>
#include <qprinter.h>

#include "ddplot.h"


class QPlotWidget : public QWidget
{
  Q_OBJECT

public:
  QString FName;
  QColor zColorLayer[MAX_ATOMS][2];  // colors of the atomic layers (0=foreground, 1=background)

  double xyzRel[MAX_ATOMS][3],xyzInit[MAX_ATOMS][3];     // coordinates of atoms
  double aDisp[MAX_ATOMS][3];                           // atomic displacements (real values)
  double xMin,yMin,zMin,xMax,yMax,zMax;
  double xSize,ySize,xCore,yCore;
  double lenBurgers;       // length of the Burgers vector
  double factor;
  double ProjVector[3];    // vector into whose direction the displacements are to be projected
                           // in the system [1-21], [-101], [111]
  double arrAngle;         // angle of arrows
  double ZFact;            // zoom factor
  double xPan,yPan;        // panning
  double DScaleFact;       // scaling factor for displacements (arrow lengths)
  double zCoordLayer[MAX_ATOMS];   // z-coordinates of layers (0 to NumZLayers-1)
  double AtomDispScale;    // scaling of atomic displacements in the relaxed configuration
  int atomScreen[MAX_ATOMS][2];    // coordinates of atoms on the screen [pixels]
  int xBorder,yBorder,xOffset,yOffset;
  int xCen,yCen,zCen;
  int zLayer[MAX_ATOMS];   // identifiers of the layers
  int NRel;                // number of atoms in the relaxed block
  int NInit;               // number of atoms in the initial block
  int aNeighbors[MAX_ATOMS][MAX_NEIGHBORS];   // list of neigbors of the atom given by the first index
  int NumZLayers;          // number of layers of atoms
  int zLineThickLayer[MAX_ATOMS];  // line thickness of atoms in a specific layer
  int zDiamLayer[MAX_ATOMS];  // diameter of atoms in a specific layer in pixels
  int shortestArrow;    // shortest arrow to be plotted (in pixels)
  int thicknessArrow;   // thickness of an arrow (in pixels)
  int atomType[MAX_ATOMS];    // flag recognizing the types of atoms
  int plotType;               // type of plot
  int DispComponent;          // component of the displacements plotted
  int AtomPos;             // plotting of atomic positions (RELAXED, UNRELAXED)  
  bool PrintMagDisp;       // print magnitudes of displacements (value/a) ?
  bool AtomNumbers;        // print the numbers of atoms
  bool isSelected;         // true if this plot is selected
  bool DisloCenter;        // show the dislocation center ?

  explicit QPlotWidget( QWidget *parent=0, const char *name=0, WFlags f=0 );
  void paintEvent( QPaintEvent * );
  void plotEdgeComponent(QPainter *p);
  void plotScrewComponent(QPainter *p);

public:
  void drawIt(QPainter *p);
  void printIt(QPrinter *printer);

  bool DrawArrow( QPainter *p, double x0, double y0, double x1, double y1 );
  void DrawCircle( QPainter *p, int atom, double x, double y, int dia);
  void DrawLine( QPainter *p, double x0, double y0, double x1, double y1, int addx, int addy);
  void DrawText( QPainter *p, double x, double y, QString txt );
  void Pan( double xsteps, double ysteps );
  void SetGeometry();
  void Zoom( double zfact );

public:
  int FindNearestAtom( int x, int y );
  bool FindNeighbors();
  bool LoadFile( QString fname );
  void PrepareBlock();
};

#endif
